class appserver::appengine::php () {

  $remi_gpg_path = '/etc/pki/rpm-gpg/RPM-GPG-KEY-remi'

  file { $remi_gpg_path:
    ensure  => present,
    owner   => 'root',
    group   => 'root',
    mode    => '0644',
    source => 'puppet:///modules/appserver/rpm-gpg-keys/remi',
  }

  exec { 'import-remi-gpg-path':
    path      => '/bin:/usr/bin:/sbin:/usr/sbin',
    command   => "rpm --import ${remi_gpg_path}",
    unless    => "rpm -q gpg-pubkey-$(echo $(gpg --throw-keyids < ${remi_gpg_path}) | cut --characters=11-18 | tr '[A-Z]' '[a-z]')",
    require   => File[$remi_gpg_path],
    logoutput => 'on_failure',
  }

  yumrepo { 'remi':
    baseurl     => 'http://rpms.famillecollet.com/enterprise/$releasever/remi/$basearch/',
    mirrorlist  => 'http://rpms.famillecollet.com/enterprise/$releasever/remi/mirror',
    enabled     => 1,
    gpgcheck    => 1,
    gpgkey      => "file://${remi_gpg_path}",
    descr       => 'Les RPM de remi pour Enterprise Linux $releasever - $basearch',
    require     => [Exec['import-remi-gpg-path'], ],
  }

  yumrepo { 'remi-php55':
    baseurl     => 'http://rpms.famillecollet.com/enterprise/$releasever/php55/$basearch/',
    mirrorlist  => 'http://rpms.famillecollet.com/enterprise/$releasever/php55/mirror',
    enabled     => 1,
    gpgcheck    => 1,
    gpgkey      => "file://${remi_gpg_path}",
    descr       => 'Les RPM de remi de PHP 5.5 pour Enterprise Linux $releasever - $basearch',
    require     => [Exec['import-remi-gpg-path'], Yumrepo['remi'], ],
  }

  php::ini { '/etc/php.ini':
    date_timezone    => 'America/Los_Angeles',
    notify => [Class['php::cli'], Class['php::fpm::daemon'], ],
  }

  class { 'php::cli':
    require => [Yumrepo['remi-php55'], ],
  }

  class { 'php::fpm::daemon':
    require => [Yumrepo['remi-php55'], ],
  }

  php::fpm::conf { 'www':
    listen  => "localhost:${appserver::params::php_default_port}",
    user    => $appserver::params::reverseproxy_default_engine,
    #TODO: Determine a better way to ensure user exists. ----- @@rhbecker
    require => [Class['::nginx'], ],
  }

}
