# TODO: Distinguish app_name in order to support multiple deployments of a single app. ----- @@rhbecker
define appserver::appsource::deployment (
  $app_name   = $title,
  $app_type,
  $dn,
  $vhost,
  $port = undef,
  $app_envvars = {},
  $vcs_engine = '',
  $vcs_origin = '',
  $vcs_origin_account = '',
  $source_ref = '',
  $reponame = '',
  $proxied_uri = '',
  $proxied_uri_static_assets = '',
  $static_assets_dir = '',
  $authn_mechanism = '',
  $authn_enable_lazy_sessions = false,
  $asset_deposits = {},
  $local_dev = false,
  $persist_location_name = '',
  $persist_location_user = '',
  $persist_location_pw = '',
) {

  # TODO: Could potentially be in the vortex, if source = synced-folder. ----- @@rhbecker
  $typed_app_name          = "${app_type}-${app_name}"
  $typed_app_name_dashless = regsubst($typed_app_name, '-', '_', 'G')
  $app_deploy_dir          = "${appserver::params::basedir_apps}/${typed_app_name}"

  $resolved_reponame       = $reponame ? {'' => "app-${typed_app_name}", default => $reponame,}

  # TODO: Condition this on synced-local vs vcs config. ----- @@rhbecker
  realize Appserver::Appsource::Vcs::Engine[$vcs_engine]
  realize Appserver::Appsource::Vcs::Origin[$vcs_origin]

  realize Group[$appserver::params::group_app_stewards]
  realize File["/etc/sudoers.d/${appserver::params::group_app_stewards}"]
  realize File[$appserver::params::basedir_apps_logs]
  realize File[$appserver::params::basedir_apps]

  vcsrepo { $app_deploy_dir:
    ensure    => present,
    provider  => $vcs_engine,
    source    => "${vcs_engine}@${vcs_origin}:${vcs_origin_account}/${resolved_reponame}.git",
    user      => 'root',
    group     => $appserver::params::group_app_stewards,
    revision  => $source_ref,
    require => [Appserver::Appsource::Vcs::Engine[$vcs_engine], Appserver::Appsource::Vcs::Origin[$vcs_origin], ],
  } ->

  file { $app_deploy_dir:
    ensure  => directory,
    recurse => true,
    owner   => 'root',
    group   => $appserver::params::group_app_stewards,
    mode    => '0664',
  }

  realize File[$appserver::params::basedir_asset_deposits]

  $asset_deposit_defaults = {
    base_path => $appserver::params::basedir_asset_deposits,
    owner     => $appserver::params::reverseproxy_default_engine,
    group     => $appserver::params::reverseproxy_default_engine,
    mode      => '0400',
  }
  create_resources(appserver::appsource::deposited_asset, $asset_deposits, $asset_deposit_defaults)

  case $authn_mechanism {
    'shibboleth': {
      realize Appserver::Authentication::Shibboleth[$appserver::idp_profile]
      realize Shibbolethsp::Mappedhost[$dn]
      unless $proxied_uri == '/' {
        shibbolethsp::mappedpath { $proxied_uri:
          associated_hostname  => $dn,
          enable_lazy_sessions => $authn_enable_lazy_sessions,
        }
      }
      $authn_directives = {'include' => '/etc/nginx/conf.d/boilerplate/shibboleth-activate-on-location.conf',}
    }
    default: {
      $authn_directives = {}
    }
  }

  if $static_assets_dir {
    $static_deploy_subdir = $static_assets_dir
  } elsif $local_dev {
    $static_deploy_subdir = 'app'
  } else {
    $static_deploy_subdir = 'dist'
  }

  if $proxied_uri_static_assets {
    nginx::resource::location { $proxied_uri_static_assets:
      ensure               => present,
      location             => $proxied_uri_static_assets,
      vhost                => $vhost,
      location_alias       => "${app_deploy_dir}/${static_deploy_subdir}/",
    }
  }

  case $app_type {

    'angularjs': {

      # TODO: I'd prefer these not be part of this case statement - rather, more like exec onlyif. ----- @@rhbecker
      realize Appserver::Appsource::Deploytool['npm']
      realize Appserver::Appsource::Deploytool['grunt-cli']
      realize Appserver::Appsource::Deploytool['bower']
      realize Appserver::Appsource::Deploytool['sass']

      nginx::resource::location { $proxied_uri:
        ensure               => present,
        location             => $proxied_uri,
        vhost                => $vhost,
        location_alias       => "${app_deploy_dir}/${static_deploy_subdir}/",
      }

      exec { "${typed_app_name} npm package dependencies":
        command => 'npm install --loglevel error --no-bin-links --production',
        onlyif  => "test -f ${app_deploy_dir}/package.json",
        path        => $appserver::params::automation_path,
        environment => $appserver::params::automation_environment,
        user        => $appserver::params::automation_user,
        group       => $appserver::params::group_app_stewards,
        umask       => '002',
        timeout     => 0,
        cwd         => $app_deploy_dir,
        logoutput   => 'on_failure',
        require     => [Vcsrepo[$app_deploy_dir], Class['appserver::appengine::nodejs'], ],
      }

      exec { "${typed_app_name} bower package dependencies":
        command => 'bower install --quiet --allow-root',
        onlyif  => "test -f ${app_deploy_dir}/bower.json",
        path        => $appserver::params::automation_path,
        environment => $appserver::params::automation_environment,
        user        => $appserver::params::automation_user,
        group       => $appserver::params::group_app_stewards,
        umask       => '002',
        timeout     => 0,
        cwd         => $app_deploy_dir,
        logoutput   => 'on_failure',
        require     => [Vcsrepo[$app_deploy_dir], Appserver::Appsource::Deploytool['bower'], ],
      }

      unless $local_dev {

        exec { "${typed_app_name} npm package dev-dependencies":
          command => 'npm install --loglevel error --no-bin-links',
          onlyif  => "test -f ${app_deploy_dir}/package.json",
          path        => $appserver::params::automation_path,
          environment => $appserver::params::automation_environment,
          user        => $appserver::params::automation_user,
          group       => $appserver::params::group_app_stewards,
          umask       => '002',
          timeout     => 0,
          cwd         => $app_deploy_dir,
          logoutput   => 'on_failure',
          require     => [Vcsrepo[$app_deploy_dir], Class['appserver::appengine::nodejs'], ],
        } ->

        exec { "${typed_app_name} grunt build":
          command => 'grunt build',
          onlyif  => "test -f ${app_deploy_dir}/Gruntfile.js",
          path        => $appserver::params::automation_path,
          environment => $appserver::params::automation_environment,
          user        => $appserver::params::automation_user,
          group       => $appserver::params::group_app_stewards,
          umask       => '002',
          timeout     => 0,
          cwd         => $app_deploy_dir,
          logoutput   => 'on_failure',
          require     => [Vcsrepo[$app_deploy_dir], Appserver::Appsource::Deploytool['grunt-cli'], Appserver::Appsource::Deploytool['sass'], Exec["${typed_app_name} npm package dependencies"], ],
        }

      }

    }

    'nodejs': {

      include appserver::appengine::nodejs
      realize Appserver::Appsource::Deploytool['npm']
      realize Package['upstart']
      realize Appserver::Appsource::Deploytool['bower']

      file { "/etc/init/${typed_app_name_dashless}.conf":
        ensure  => present,
        content => template('appserver/upstart-application.conf.erb'),
        owner   => 'root',
        group   => 'root',
        mode    => '0644',
      }

      service { $typed_app_name_dashless:
        ensure     => running,
        hasstatus  => true,
        hasrestart => true,
        start      => "start ${typed_app_name_dashless}",
        stop       => "stop ${typed_app_name_dashless}",
        status     => "status ${typed_app_name_dashless} | grep '/running' 1>/dev/null 2>&1",
        subscribe  => File["/etc/init/${typed_app_name_dashless}.conf"],
      }

      nginx::resource::upstream { $typed_app_name_dashless:
        ensure  => present,
        members => ["localhost:${port}", ],
      }

      nginx::resource::location { $proxied_uri:
        ensure               => present,
        location             => $proxied_uri,
        vhost                => $vhost,
        proxy                => "http://${typed_app_name_dashless}",
        location_cfg_prepend => $authn_directives,
      }

      exec { "${typed_app_name} npm package dependencies":
        command => 'npm install --loglevel error --no-bin-links --production',
        onlyif  => "test -f ${app_deploy_dir}/package.json",
        path        => $appserver::params::automation_path,
        environment => $appserver::params::automation_environment,
        user        => $appserver::params::automation_user,
        group       => $appserver::params::group_app_stewards,
        umask       => '002',
        timeout     => 0,
        cwd         => $app_deploy_dir,
        logoutput   => 'on_failure',
        require     => [Vcsrepo[$app_deploy_dir], Class['appserver::appengine::nodejs'], ],
      }

      exec { "${typed_app_name} bower package dependencies":
        command => 'bower install --quiet --allow-root',
        onlyif  => "test -f ${app_deploy_dir}/bower.json",
        path        => $appserver::params::automation_path,
        environment => $appserver::params::automation_environment,
        user        => $appserver::params::automation_user,
        group       => $appserver::params::group_app_stewards,
        umask       => '002',
        timeout     => 0,
        cwd         => $app_deploy_dir,
        logoutput   => 'on_failure',
        require     => [Vcsrepo[$app_deploy_dir], Appserver::Appsource::Deploytool['bower'], ],
      }

      Exec["${typed_app_name} npm package dependencies"]->Service[$typed_app_name_dashless]

    }

    'php-symfony': {

      include appserver::appengine::php
      realize Appserver::Appsource::Deploytool['composer']

      php::module { [ 'pdo', 'mbstring', 'intl', 'pecl-apcu', 'xml', 'process', 'opcache', 'mysql', ]:
        require => [Yumrepo['remi-php55'], ],
        notify => [Class['php::fpm::daemon'], ],
      }

      php::module::ini { 'pecl-apcu':
        settings => {
          'apc.enabled'        => '1',
          'apc.shm_size'       => '64M',
          'apc.enable_cli'     => '0',
          'apc.serializer'     => 'php',
          'apc.mmap_file_mask' => '/tmp/apc.XXXXXX',
        },
        require => [Yumrepo['remi-php55'], ],
        notify => [Class['php::fpm::daemon'], ],
      }

      nginx::resource::upstream { $typed_app_name_dashless:
        ensure  => present,
        members => ["localhost:${appserver::params::php_default_port}", ],
      }

      nginx::resource::location { "${dn}_location_${proxied_uri}":
        ensure               => present,
        location             => $proxied_uri,
        vhost                => $vhost,
        www_root             => "${app_deploy_dir}/web",
        location_cfg_append  => {'try_files' => "\$uri @fcrewrite-${typed_app_name}", },
      }

      nginx::resource::location { "@fcrewrite: ${typed_app_name}":
        ensure               => present,
        location             => "@fcrewrite-${typed_app_name}",
        vhost                => $vhost,
        www_root             => "${app_deploy_dir}/web",
        location_cfg_append  => {'rewrite' => '^(.*)$ /app.php/$1 last',},
      }

      nginx::resource::location { "${dn}_location_send-all-to-php":
        ensure               => present,
        location             => '~ ^/(app|app_dev|config)\.php(/|$)',
        vhost                => $vhost,
        fastcgi              => $typed_app_name_dashless,
        fastcgi_split_path   => '^(.+\.php)(/.*)$',
        fastcgi_params       => '/etc/nginx/fastcgi_params',
        location_cfg_prepend => {'fastcgi_param' => 'HTTPS off'},
        location_cfg_append  => {'fastcgi_param' => 'SCRIPT_FILENAME $document_root$fastcgi_script_name',},
        www_root             => "${app_deploy_dir}/web",
      }

      file { ["${app_deploy_dir}/app/logs", "${app_deploy_dir}/app/cache",]:
        ensure  => directory,
        owner   => 'nginx',
        require => [File[$app_deploy_dir], ],
      }

      # TODO: Replace this hack with something less ridiculous. ----- @@rhbecker
      file { '/var/www/private':
        ensure => directory,
      }
      File['/var/www/private'] -> Appserver::Appsource::Deposited_asset<| base_path == '/var/www/private' |>

      composer::exec { "composer install: ${app_deploy_dir}":
        cmd     => 'install',
        cwd     => $app_deploy_dir,
        scripts => true,
        prefer_source => true,
        require => [Appserver::Appsource::Deploytool['composer'], ],
      }

      # TODO: Condition this block: local or remote persistence? ----- @@rhbecker
      realize Appserver::Persistence::Relationaldb['mysql']
      mysql::db { $persist_location_name:
        user     => $persist_location_user,
        password => $persist_location_pw,
        host     => ['localhost',],
        grant    => ['all'],
        require  => Appserver::Persistence::Relationaldb['mysql'],
      }

      # TODO: Condition this block: persisted data is ephemeral? ----- @@rhbecker
      $symfony_app_bootstrap_script = 'app/bootstrap.application.sh'
      file { "${app_deploy_dir}/${symfony_app_bootstrap_script}":
        ensure  => file,
        mode   => '0764',
        require => [Vcsrepo[$app_deploy_dir], File[$app_deploy_dir], ],
      }
      exec { "${typed_app_name} bootstrap":
        command => "bash ${symfony_app_bootstrap_script}",
        path        => $appserver::params::automation_path,
        environment => $appserver::params::automation_environment,
        user        => $appserver::params::automation_user,
        group       => $appserver::params::group_app_stewards,
        umask       => '002',
        timeout     => 0,
        cwd         => $app_deploy_dir,
        logoutput   => 'on_failure',
        require => [File["${app_deploy_dir}/${symfony_app_bootstrap_script}"], Appserver::Persistence::Relationaldb['mysql'], Mysql::Db[$persist_location_name], Package['php-cli'], Composer::Exec["composer install: ${app_deploy_dir}"], ],
      }

    }

  }

  Appserver::Appsource::Vcs::Engine[$vcs_engine] -> Appserver::Appsource::Vcs::Origin[$vcs_origin]
  File[$app_deploy_dir] -> Appserver::Appsource::Deposited_asset<| base_path == $app_deploy_dir |>
}
