define appserver::http::hosteddomain(
  $dn                     = $title,
  $sslcert                = '',
  $sslkey                 = '',
  $http_redirect          = true,
  $noncanonical_hostnames = [],
  $local_dev              = false,
  $enable_lazy_sessions   = true,
  $apps                   = {},
){

  appserver::http::sslcert{ $dn:
    cert => $sslcert,
    key  => $sslkey,
  }

  realize Nginx::Resource::Logformat['canonical']

  nginx::resource::vhost { "canonical:${dn}":
    ensure               => present,
    server_name          => [$dn,],
    access_log           => "/var/log/nginx/canonical:${dn}.access.log canonical",
    error_log            => "/var/log/nginx/canonical:${dn}.error.log crit",
    include_files        => ['/etc/nginx/conf.d/boilerplate/shibboleth-sp-locations.conf', ],
    listen_port          => '443',
    ssl_cert             => "${appserver::params::dir_conf_http_ssl}/${dn}.chained.crt",
    ssl_key              => "${appserver::params::dir_conf_http_ssl}/${dn}.key",
    ssl                  => true,
    ssl_protocols        => 'TLSv1 TLSv1.1 TLSv1.2',
    use_default_location => false,
  }

  if $http_redirect {
    appserver::http::noncanonical_hostname { $dn:
      canonical => $dn,
      ssl       => false,
    }
  }

  appserver::http::noncanonical_hostname { $noncanonical_hostnames:
    canonical => $dn,
  }

  $defaults_app = {
    'dn'                 => $dn,
    'vhost'              => "canonical:${dn}",
    'vcs_engine'         => 'git',
    'vcs_origin'         => 'bitbucket.org',
    'vcs_origin_account' => 'oris',
    'source_ref'         => 'master',
    'local_dev'          => $local_dev,
  }
  create_resources(appserver::appsource::deployment, $apps, $defaults_app)

  # TODO: Stop assuming hosts will require shib-authn. ----- @@rhbecker
  @shibbolethsp::mappedhost {$dn:
    enable_lazy_sessions => $enable_lazy_sessions,
  }

  Class['appserver::http::reverseproxy']
    -> Appserver::Http::Sslcert[$dn]
    -> Nginx::Resource::Vhost<| title == "canonical:${dn}" |>
    -> Appserver::Appsource::Deployment<| vhost == "canonical:${dn}" |>
}
